--
-- Jill's Multi-Video Stream smplayer2 script
-- by A. Schiffler, Oct/Nov 2007
-- last edits Feb 2008
--

--
-- Keyboard functions:
--  1,2 - select screen to adjust
--  space - select frame to adjust
--  cursor,home - select corner to adjust
--  ESC - exit edit mode
--  o - show overlay grid
--  s - save configuration
--  q - quit player to shell
--  r - reboot player
--  t - shutdown player (power off PC)
--  p - show parallel port I/O info
--  d - toggle I/O debugging display
--
-- Mouse functions:
--  click, drag - set corner position in edit mode
--

-- load utility functions
dofile("utility50.lua")

-- === smplayer callbacks ===

function Config()
 print "Configuration Section"
 -- Setup display configuration (dual 720p screens)
 smplayer.width=1280
 smplayer.height=768
 smplayer.screens=2
 smplayer.cursor=0
 -- Setup definition file
 config_filename="jill_multi.conf"
 -- parallel port address
 --  BASEPORT0 0x03bc  /* /dev/lp0 */
 --  BASEPORT1 0x0378  /* /dev/lp1 */
 --  BASEPORT2 0x0278  /* /dev/lp2 */
 --  BASEPORT3 0xb400  /* /dev/lpX */
 baseport=0xb400
end


function Init()
 print "Initialization Section"

 -- video playback rate in Hz
 rate=25.0

 -- video stream cache (multiples of 1MB times 25 SD streams, 4MB times 6 HD streams)
 cache_scale=1.5
 
 -- to attract loop timeout seconds
 timeout=600
 timeout_counter=0
 
 -- to rewind timeout seconds
 rtimeout=3600
 rtimeout_counter=0

 -- fade time as factor of 1 second
 fade_scale=4.0
  
 -- initialize printer port, power one bit
 interface=parallel.open(baseport) 
 interface.data=1

 -- HD compression mode: dxt3, rgba512, rgba1024lzf
 hd_compression_mode="rgba1024lzf" 
 
 -- number of SD source files
 num_sd_filenames = 30
 
 -- define SD source filenames
 filename={}
 -- A set
 filename[ 1] = "/video/A1_box.dxt3";
 filename[ 2] = "/video/A2_box.dxt3";
 filename[ 3] = "/video/A3_box.dxt3";
 filename[ 4] = "/video/A4_box.dxt3";
 filename[ 5] = "/video/A5_box.dxt3";
 -- B set
 filename[ 6] = "/video/B1_box.dxt3";
 filename[ 7] = "/video/B2_box.dxt3";
 filename[ 8] = "/video/B3_box.dxt3";
 filename[ 9] = "/video/B4_box.dxt3";
 filename[10] = "/video/B5_box.dxt3";
 -- C set
 filename[11] = "/video/C1_box.dxt3";
 filename[12] = "/video/C2_box.dxt3";
 filename[13] = "/video/C3_box.dxt3";
 filename[14] = "/video/C4_box.dxt3";
 filename[15] = "/video/C5_box.dxt3";
 -- D set
 filename[16] = "/video/D1_box.dxt3";
 filename[17] = "/video/D2_box.dxt3";
 filename[18] = "/video/D3_box.dxt3";
 filename[19] = "/video/D4_box.dxt3";
 filename[20] = "/video/D5_box.dxt3";
 -- E set
 filename[21] = "/video/E1_box.dxt3";
 filename[22] = "/video/E2_box.dxt3";
 filename[23] = "/video/E3_box.dxt3";
 filename[24] = "/video/E4_box.dxt3";
 filename[25] = "/video/E5_box.dxt3";
 -- F set
 filename[26] = "/video/F1_box.dxt3";
 filename[27] = "/video/F2_box.dxt3";
 filename[28] = "/video/F3_box.dxt3";
 filename[29] = "/video/F4_box.dxt3";
 filename[30] = "/video/F5_box.dxt3";

 -- size of SD sources
 sd_width=256
 sd_height=256

 -- create 10Hz overlay pane and frame
 print "Creating overlay frame ..."
 overlay_frame = 100
 overlay_video = ft.start(overlay_frame, sd_width, sd_height, 10.0)

 -- draw blank overlay
 overlay_pane = mt.create(sd_width,sd_height)
 mt.fill(overlay_pane,"black")
 mt.clear(overlay_pane,0)
 box_size=sd_width/16
 ft.blit(overlay_video, overlay_pane, 0, 0, 0);
 overlay_video.a=0
 overlay_video.screens=1

 -- frame which is being edited
 edit_frame = 0
 edit_screen = 0
 edit_corner = 1
 edit_blink = 1
 
 -- create message frame
 print ("Creating message frame ...")
 message_frame = 101
 message_pane = mt.create(1024,64)
 message_video = ft.start(message_frame, 1024, 64, 20.0)
 message_video.x=0.05
 message_video.y=0.45
 message_video.w=0.9
 message_video.h=64/768
 
 -- text to display (initially blank)
 message=""
 message_counter=0
 
 -- IO debugging flag
 io_debug=0
  
 -- start IO readers for SD sources
 sdb=2
 --
 filetio={}
 for i=1,num_sd_filenames do
  print ("Opening SD video file '" .. filename[i] .. "' ...")
  filetio[i] = tio.open(filename[i],sdb,cache_scale*1024*1024);
  filetio[i].autorewind=1
 end
 
 -- current/initial play state
 playstate=1
 
 -- playstate transition
 playstate_wanted=0
 playstate_counter=0
  
 -- number of play states in total
 num_playstates = 6

 -- maximum number of zones
 num_frames = 25
  
 -- files playing in each playstate, per zone, by file index
 print "Assigning SD video files to playstates ..."
 playstate_file={}
 for i=1,num_playstates do playstate_file[i]={} end
 -- state 1 = A
 i=1
 playstate_file[i][ 1]=1
 playstate_file[i][ 2]=2
 playstate_file[i][ 3]=3
 playstate_file[i][ 4]=4
 playstate_file[i][ 5]=5
 --
 playstate_file[i][ 6]=5
 playstate_file[i][ 7]=4
 playstate_file[i][ 8]=2
 playstate_file[i][ 9]=1
 playstate_file[i][10]=3
 --
 playstate_file[i][11]=3
 playstate_file[i][12]=1
 playstate_file[i][13]=5
 playstate_file[i][14]=4
 playstate_file[i][15]=2
 --
 playstate_file[i][16]=5
 playstate_file[i][17]=4
 playstate_file[i][18]=2
 playstate_file[i][19]=3
 playstate_file[i][20]=1
 --
 playstate_file[i][21]=1
 playstate_file[i][22]=3
 playstate_file[i][23]=5
 playstate_file[i][24]=2
 playstate_file[i][25]=4
 -- state 2 = B
 i=2
 offset=5
 playstate_file[i][ 1]=offset+1
 playstate_file[i][ 2]=offset+2
 playstate_file[i][ 3]=offset+3
 playstate_file[i][ 4]=offset+4
 playstate_file[i][ 5]=offset+5
 --
 playstate_file[i][ 6]=offset+5
 playstate_file[i][ 7]=offset+4
 playstate_file[i][ 8]=offset+2
 playstate_file[i][ 9]=offset+1
 playstate_file[i][10]=offset+3
 --
 playstate_file[i][11]=offset+3
 playstate_file[i][12]=offset+1
 playstate_file[i][13]=offset+5
 playstate_file[i][14]=offset+4
 playstate_file[i][15]=offset+2
 --
 playstate_file[i][16]=offset+5
 playstate_file[i][17]=offset+4
 playstate_file[i][18]=offset+2
 playstate_file[i][19]=offset+3
 playstate_file[i][20]=offset+1
 --
 playstate_file[i][21]=offset+1
 playstate_file[i][22]=offset+3
 playstate_file[i][23]=offset+5
 playstate_file[i][24]=offset+2
 playstate_file[i][25]=offset+4
 -- state 3 = C
 i=3
 offset=10
 playstate_file[i][ 1]=offset+1
 playstate_file[i][ 2]=offset+2
 playstate_file[i][ 3]=offset+3
 playstate_file[i][ 4]=offset+4
 playstate_file[i][ 5]=offset+5
 --
 playstate_file[i][ 6]=offset+5
 playstate_file[i][ 7]=offset+4
 playstate_file[i][ 8]=offset+2
 playstate_file[i][ 9]=offset+1
 playstate_file[i][10]=offset+3
 --
 playstate_file[i][11]=offset+3
 playstate_file[i][12]=offset+1
 playstate_file[i][13]=offset+5
 playstate_file[i][14]=offset+4
 playstate_file[i][15]=offset+2
 --
 playstate_file[i][16]=offset+5
 playstate_file[i][17]=offset+4
 playstate_file[i][18]=offset+2
 playstate_file[i][19]=offset+3
 playstate_file[i][20]=offset+1
 --
 playstate_file[i][21]=offset+1
 playstate_file[i][22]=offset+3
 playstate_file[i][23]=offset+5
 playstate_file[i][24]=offset+2
 playstate_file[i][25]=offset+4
 -- state 4 = D
 i=4
 offset=15
 playstate_file[i][ 1]=offset+1
 playstate_file[i][ 2]=offset+2
 playstate_file[i][ 3]=offset+3
 playstate_file[i][ 4]=offset+4
 playstate_file[i][ 5]=offset+5
 --
 playstate_file[i][ 6]=offset+5
 playstate_file[i][ 7]=offset+4
 playstate_file[i][ 8]=offset+2
 playstate_file[i][ 9]=offset+1
 playstate_file[i][10]=offset+3
 --
 playstate_file[i][11]=offset+3
 playstate_file[i][12]=offset+1
 playstate_file[i][13]=offset+5
 playstate_file[i][14]=offset+4
 playstate_file[i][15]=offset+2
 --
 playstate_file[i][16]=offset+5
 playstate_file[i][17]=offset+4
 playstate_file[i][18]=offset+2
 playstate_file[i][19]=offset+3
 playstate_file[i][20]=offset+1
 --
 playstate_file[i][21]=offset+1
 playstate_file[i][22]=offset+3
 playstate_file[i][23]=offset+5
 playstate_file[i][24]=offset+2
 playstate_file[i][25]=offset+4
 -- state 5 = E
 i=5
 offset=20
 playstate_file[i][ 1]=offset+1
 playstate_file[i][ 2]=offset+2
 playstate_file[i][ 3]=offset+3
 playstate_file[i][ 4]=offset+4
 playstate_file[i][ 5]=offset+5
 --
 playstate_file[i][ 6]=offset+5
 playstate_file[i][ 7]=offset+4
 playstate_file[i][ 8]=offset+2
 playstate_file[i][ 9]=offset+1
 playstate_file[i][10]=offset+3
 --
 playstate_file[i][11]=offset+3
 playstate_file[i][12]=offset+1
 playstate_file[i][13]=offset+5
 playstate_file[i][14]=offset+4
 playstate_file[i][15]=offset+2
 -- 
 playstate_file[i][16]=offset+5
 playstate_file[i][17]=offset+4
 playstate_file[i][18]=offset+2
 playstate_file[i][19]=offset+3
 playstate_file[i][20]=offset+1
 --
 playstate_file[i][21]=offset+1
 playstate_file[i][22]=offset+3
 playstate_file[i][23]=offset+5
 playstate_file[i][24]=offset+2
 playstate_file[i][25]=offset+4
 -- state 6 = F (attract)
 i=6
 offset=25
 playstate_file[i][ 1]=offset+1
 playstate_file[i][ 2]=offset+2
 playstate_file[i][ 3]=offset+3
 playstate_file[i][ 4]=offset+4
 playstate_file[i][ 5]=offset+5
 --
 playstate_file[i][ 6]=offset+5
 playstate_file[i][ 7]=offset+4
 playstate_file[i][ 8]=offset+2
 playstate_file[i][ 9]=offset+1
 playstate_file[i][10]=offset+3
 --
 playstate_file[i][11]=offset+3
 playstate_file[i][12]=offset+1
 playstate_file[i][13]=offset+5
 playstate_file[i][14]=offset+4
 playstate_file[i][15]=offset+2
 --
 playstate_file[i][16]=offset+5
 playstate_file[i][17]=offset+4
 playstate_file[i][18]=offset+2
 playstate_file[i][19]=offset+3
 playstate_file[i][20]=offset+1
 --
 playstate_file[i][21]=offset+1
 playstate_file[i][22]=offset+3
 playstate_file[i][23]=offset+5
 playstate_file[i][24]=offset+2
 playstate_file[i][25]=offset+4

 -- calculate frame-clone matrix from filename assignments
 print "Calculating frame-clone matrix ..."
 playstate_clone={}
 for i=1,num_playstates do
  playstate_clone[i]={}
  for j=1,num_frames do
   -- reset to non-cloned
   playstate_clone[i][j]=0;
   -- look for this file index
   search_file=playstate_file[i][j];
   -- scan from 2nd file
   if (j>1) then
    clone_from_frame=0
    for k=1,(j-1) do
     if (playstate_file[i][k]==search_file) then
      if (playstate_clone[i][k]==0) then
       clone_from_frame=k
       k=j
      end
     end
    end
    if (clone_from_frame>0) then
     playstate_clone[i][j]=clone_from_frame
    end
   end
  end
 end

 -- create SD frames and readers for all playstates
 print "Creating display frames ..."
 playstate_video={}
 for i=1,num_frames do
  -- create new frame
  playstate_video[i]=ft.start(i, sd_width, sd_height, rate);
  -- lock frame to screen 1
  playstate_video[i].screens=1;
  -- default position
  dx=(1280-720)/1280
  ss=0.154
  default_x=math.mod((i-1),5)*(1.0-dx)/5.0+dx/2
  default_y=math.floor((i-1)/5.0)*1.0/5.0
  default_w=ss*(720.0/1280.0)
  default_h=ss
  -- position frame
  playstate_video[i].x=default_x
  playstate_video[i].y=default_y
  playstate_video[i].w=default_w
  playstate_video[i].h=default_h
 end 

 -- maybe clone/unclone the frame
 print ("Setting up cloning ...")
 for i=1,num_frames do
  if (playstate_clone[playstate][i]>0) then
   ft.clone(playstate_video[ playstate_clone[playstate][i] ], playstate_video[i]);  
  else
   ft.unclone(playstate_video[i]);    
  end
 end

 -- load to config file
 if (file_exists(config_filename)) then
  print ("Reading configuration file ...")
  f=io.open(config_filename,"r")
  t = f:read("*all")
  f:close()
  data=unpickle(t)
  globals.setCorners(1,data[100])
  globals.setCorners(2,data[101])
  for i=1,num_frames do
   ft.setCorners(i,data[i])
  end
 end

 -- define HD sourcefilenames 
 print ("Creating HD video ...")
 num_hd_filenames=6
 hdfilename={}
 if (hd_compression_mode=="dxt3") then
  hdfilename[1] ="/video/HD_A_hd.dxt3"
  hdfilename[2] ="/video/HD_B_hd.dxt3"
  hdfilename[3] ="/video/HD_C_hd.dxt3"
  hdfilename[4] ="/video/HD_D_hd.dxt3"
  hdfilename[5] ="/video/HD_E_hd.dxt3"
  hdfilename[6] ="/video/HD_F_hd.dxt3"
 end
 if (hd_compression_mode=="rgba512") then
  hdfilename[1] ="/video/HD_A_hd.rgba512"
  hdfilename[2] ="/video/HD_B_hd.rgba512"
  hdfilename[3] ="/video/HD_C_hd.rgba512"
  hdfilename[4] ="/video/HD_D_hd.rgba512"
  hdfilename[5] ="/video/HD_E_hd.rgba512"
  hdfilename[6] ="/video/HD_F_hd.rgba512"
 end
 if (hd_compression_mode=="rgba1024lzf") then
  hdfilename[1] ="/video/HD_A_hd.rgba1024lzf"
  hdfilename[2] ="/video/HD_B_hd.rgba1024lzf"
  hdfilename[3] ="/video/HD_C_hd.rgba1024lzf"
  hdfilename[4] ="/video/HD_D_hd.rgba1024lzf"
  hdfilename[5] ="/video/HD_E_hd.rgba1024lzf"
  hdfilename[6] ="/video/HD_F_hd.rgba1024lzf"
 end

 -- which frame is playing HD
 hd_frame=99
 
 -- setup HD frame
 if (hd_compression_mode=="dxt3") then
  hd_video = ft.start(hd_frame, 1280, 720, rate) 
 end
 if (hd_compression_mode=="rgba512") then
  hd_video = ft.start(hd_frame, 512, 512, rate) 
 end
 if (hd_compression_mode=="rgba1024lzf") then
  hd_video = ft.start(hd_frame, 1024, 512, rate) 
 end
 hd_video.x=0.0
 hd_video.y=0.0
 hd_video.screens=2

 -- setup HD streams
 hdb=8
 --
 hd_file={}
 for i=1,num_hd_filenames do
  print ("Opening HD file " .. hdfilename[i] .. "...")
  hd_file[i] = tio.open(hdfilename[i],hdb,cache_scale*4*1024*1024)
  hd_file[i].autorewind=1
 end

 -- assign hd video to playstate
 playstate_hdfile={}
 playstate_hdfile[1]=1
 playstate_hdfile[2]=2
 playstate_hdfile[3]=3
 playstate_hdfile[4]=4
 playstate_hdfile[5]=5
 playstate_hdfile[6]=6
 
 print ("File and frame setup done.")
 
end

function Redraw(frame)
 if (frame==hd_frame) then
  -- update HD video
  ft.s3tc_tioread(hd_video,hd_file[ playstate_hdfile[playstate] ]);
  ft.update(hd_video);

 elseif (frame==message_frame) then
  -- fade message out
  if (message_counter>0) then
   message_counter=message_counter-1   
   if (message_counter<10) then
    a=(message_counter/10.0)
    message_video.a=a*a
   end
  end
  -- create message
  if (string.len(message)>0) then
   message_counter=20
   message_video.a=1.0
   mt.fill(message_pane,"black")
   mt.clear(message_pane,192)
   message_image = mt.text(message,"arial.ttf",32,1, "yellow")
   mt.blit(message_pane, message_image, 16, 16 ,1)
   ft.blit(message_video, message_pane, 0, 0, 0)
   ft.update(message_video)
   collectgarbage()
   message=""
  end
 
 -- overlay frame for setup
 elseif (frame==overlay_frame) then
  -- draw overlay [X]
  mt.fill(overlay_pane,"black")
  mt.clear(overlay_pane,0)
  box_size=sd_width/16
  box_color="white"
  if (edit_corner==1) then
   box_color="red"
  end
  mt.box(overlay_pane, 0 ,0, box_size, box_size, box_color)
  box_color="white"
  if (edit_corner==2) then
   box_color="red"
  end
  mt.box(overlay_pane, sd_width-box_size, 0, sd_width, box_size, box_color)
  box_color="white"
  if (edit_corner==3) then
   box_color="red"
  end
  mt.box(overlay_pane, sd_width-box_size, sd_height-box_size, sd_width, sd_height, box_color)
  box_color="white"
  if (edit_corner==4) then
   box_color="red"
  end
  mt.box(overlay_pane, 0, sd_height-box_size, box_size, sd_height, box_color)
  if (edit_blink==1) then
   box_color="white"
   if (edit_corner==5) then
    box_color="red"
   end
   edit_blink=0
  else
   box_color="black"
   edit_blink=1
  end
  mt.box(overlay_pane, (sd_width-box_size)/2, (sd_height-box_size)/2, (sd_width+box_size)/2, (sd_height+box_size)/2, box_color)
  ft.blit(overlay_video, overlay_pane, 0, 0, 0);

  -- make overlay visible on the right screen or frame
  if ((edit_frame==0) and (edit_screen==0)) then
   overlay_video.a=0
  else
   if (edit_frame>0) then
    -- make visible on screen 1
    overlay_video.screens=1
    overlay_video.a=1.0
    --place over edit frame
    corners = ft.getCorners(edit_frame)  
    ft.setCorners(overlay_frame,corners)
   elseif (edit_screen>0) then
    -- make visible over edit screen
    overlay_video.screens=edit_screen
    overlay_video.a=1.0
    -- cover whole screen
    overlay_video.x=0.0
    overlay_video.y=0.0
    overlay_video.w=1.0
    overlay_video.h=1.0
   end 
  end
  ft.update(overlay_video);

 elseif (frame<=num_frames) then
  -- map frame to video file of playstate
  fileindex = playstate_file[playstate][frame]

  -- update SD content if we are not cloning
  if (playstate_clone[playstate][frame]==0) then
   -- print ("Reading for frame " .. frame .. " from file " .. fileindex)
   ft.s3tc_tioread(playstate_video[frame], filetio[ fileindex ]);
   ft.update( playstate_video[frame] );
  end
 end
 
 -- tie playstate transitions to frame 1
 if (frame==1) then
  if ((playstate_counter>0) and (playstate_wanted>0)) then
   -- transition countdown
   playstate_counter=playstate_counter-1
   -- which transition sequence
   if (playstate_wanted~=playstate) then
    -- fade out
    a=playstate_counter/(fade_scale*rate)
    for i=1,num_frames do
     playstate_video[i].a=a
    end
    hd_video.a=a
    -- switch?
    if (playstate_counter==0) then
     -- change playstate
     playstate=playstate_wanted
     playstate_counter=fade_scale*rate
     -- maybe clone/unclone the frames
     for i=1,num_frames do
      if (playstate_clone[playstate][i]>0) then
       ft.clone(playstate_video[ playstate_clone[playstate][i] ], playstate_video[i]);  
      else
       ft.unclone(playstate_video[i]);    
      end
     end
    end
   else
    -- fade in
    a=1.0-playstate_counter/(fade_scale*rate)
    for i=1,num_frames do
     playstate_video[i].a=a
    end
    hd_video.a=a
    -- mark transition as finished
    if (playstate_counter==0) then
     playstate_wanted=0
    end
   end   
  end
 end
 
end

function Key(char,sym)
 -- Debug
 -- print ("Key "..char.." pressed char="..string.byte(char) .. " sym="..sym)

 -- O: toggle frame outline
 if (sym==111) then
  if (smplayer.outline==0) then
   message="Outline on ..."
   smplayer.outline = 1
  else
   message="Outline off ..."
   smplayer.outline = 0
  end
 end

 -- P: display port status
 if (sym==112) then
  s=interface.status
  s1=((s & 8)~=8)
  s2=((s & 64)~=64)
  s3=((s & 128)==128)
  s4=((s & 32)~=32)
  s5=((s & 16)~=16)
  message="Interface: "
  if s1 then message=message.." S1" end
  if s2 then message=message.." S2" end
  if s3 then message=message.." S3" end
  if s4 then message=message.." S4" end
  if s5 then message=message.." S5" end
  if (s1 or s2 or s3 or s4 or s5) then
   message=message.." on"
  else
   message=message.." ALL off"
  end
 end
 
 -- Space: frame selector
 if (sym==32) then
  edit_screen=0
  edit_frame=edit_frame+1
  edit_corner=5
  if (edit_frame>num_frames) then
   edit_frame=1
  end
  smplayer.cursor=1
  message="Editing frame " .. edit_frame .. " ..."
 end

 -- Number Keys: screen selector
 if (sym==49) then
  edit_screen=1
  edit_frame=0
  smplayer.cursor=1
  message="Editing screen 1 ..."
 end
 if (sym==50) then
  edit_screen=2
  edit_frame=0
  smplayer.cursor=1
  message="Editing screen 2 ..."
 end

 -- Selection of edit position for setup mode

 -- Cursor: up
 if (sym==273) then
  if (edit_corner==4) then
   edit_corner=1
  elseif (edit_corner==3) then
   edit_corner=2
  elseif (edit_corner==5) then
   edit_corner=1
  end
  message="Editing corner "..edit_corner.." ..."
 end

 -- Cursor: right
 if (sym==275) then
  if (edit_corner==1) then 
   edit_corner=2
  elseif (edit_corner==4) then
   edit_corner=3
  elseif (edit_corner==5) then
   edit_corner=2
  end
  message="Editing corner "..edit_corner.." ..."
 end

 -- Cursor: down
 if (sym==274) then
  if (edit_corner==1) then
   edit_corner=4
  elseif (edit_corner==2) then
   edit_corner=3
  elseif (edit_corner==5) then
   edit_corner=4
  end
  message="Editing corner "..edit_corner.." ..."
 end

 -- Cursor: left
 if (sym==276) then
  if (edit_corner==2) then
   edit_corner=1
  elseif (edit_corner==3) then
   edit_corner=4
  elseif (edit_corner==5) then
   edit_corner=1
  end
  message="Editing corner "..edit_corner.." ..."
 end

 -- Home: set center editing
 if (sym==278) then
  edit_corner=5
  message="Editing center ..."
 end
 
 -- ESC: clear selection
 if (sym==27) then
  if ((edit_frame>0) or (edit_screen>0)) then
   edit_frame=0
   edit_screen=0
   edit_corner=1
   smplayer.cursor=0
   message="Exiting edit mode ..."
  end
 end

 -- F1-F6: playstate switch
 if ((sym>=282) and (sym<=(287))) then
  possible_playstate=sym-281
  if (possible_playstate~=playstate) then
   playstate_wanted=possible_playstate
   if (playstate_counter==0) then 
    playstate_counter=fade_scale*rate 
   end
   message="Switching to playstate "..playstate_wanted.." ..."
   timeout_counter=0
   rtimeout_counter=0
  end
 end

 -- S: save current geometric configuration
 if (sym==115) then
  -- create config array
  data={}
  data[100] = smplayer.getCorners(1)
  data[101] = smplayer.getCorners(2)
  for i=1,num_frames do
   data[i] = ft.getCorners(i)
  end
  -- save to config file
  f=io.open(config_filename,"w+")
  f:write(pickle(data))
  f:close()
  message="Configuration saved ..."
 end

 -- Q: quit player
 if (sym==113) then
  smplayer.quit()
 end

 -- R: reboot player
 if (sym==114) then
  os.exit(1)
 end

 -- T: shutdown player
 if (sym==116) then
  os.exit(2)
 end

 -- D: toggle I/O debugging display
 if (sym==100) then
  if (io_debug==1) then
   io_debug=0
   message="I/O debug OFF" 
  else
   io_debug=1
   message="I/O debug ON" 
  end
 end
 
end

function Mouse(screen,x,y,t)
 -- warp screen
 if (edit_screen>0) then
  if (edit_screen==screen) then
   corners = smplayer.getCorners(edit_screen)
   if (edit_corner==1) then
    corners[1]=x
    corners[2]=y
   elseif (edit_corner==2) then
    corners[3]=x
    corners[4]=y
   elseif (edit_corner==3) then
    corners[5]=x
    corners[6]=y
   elseif (edit_corner==4) then
    corners[7]=x
    corners[8]=y
   elseif (edit_corner==5) then
    -- no shift
   end   
   smplayer.setCorners(edit_screen, corners)
  end
 end
 if  (edit_frame>0) then
  -- reposition frame
  corners = ft.getCorners(edit_frame)  
  if (edit_corner==1) then
   if ((x<corners[5]) and (y<corners[6])) then
    corners[1]=x
    corners[2]=y
    corners[7]=x
    corners[4]=y
   end
  elseif (edit_corner==2) then
   if ((x>corners[7]) and (y<corners[8])) then
    corners[3]=x
    corners[4]=y
    corners[5]=x
    corners[2]=y
   end
  elseif (edit_corner==3) then
   if ((x>corners[1]) and (y>corners[2])) then
    corners[5]=x
    corners[6]=y
    corners[3]=x
    corners[8]=y
   end
  elseif (edit_corner==4) then
   if ((x<corners[3]) and (y>corners[4])) then
    corners[7]=x
    corners[8]=y
    corners[1]=x
    corners[6]=y
   end
  elseif (edit_corner==5) then
   -- calculate center
   cx=corners[1]+(corners[3]-corners[1])/2
   cy=corners[2]+(corners[8]-corners[2])/2
   -- calculate shift from center
   dx=x-cx
   dy=y-cy
   -- adjust by shifting
   corners[1]=corners[1]+dx   
   corners[2]=corners[2]+dy  
   corners[3]=corners[3]+dx   
   corners[4]=corners[4]+dy  
   corners[5]=corners[5]+dx   
   corners[6]=corners[6]+dy  
   corners[7]=corners[7]+dx   
   corners[8]=corners[8]+dy  
  end   
  corners = ft.setCorners(edit_frame,corners)
 end
end

function Idle(ticks)
 -- second-interval state change
 seconds=math.floor(ticks/1000)
 -- second ticks
 if (last_seconds~=seconds) then
  -- query parallel port bits
  s=interface.status
  s1=((s & 8)~=8)
  s2=((s & 64)~=64)
  s3=((s & 128)==128)
  s4=((s & 32)~=32)  
  s5=((s & 16)~=16)
  -- Generate 1-5 playstate
  s=0
  if s1 then s=s+1 end
  if s2 then s=s+2 end
  if s3 then s=s+4 end
  if s4 then s=s+8 end
  if (s>5) then
   s=0
  end
  -- do we have any triggers
  if ((s>0) or s5) then
   -- set playstate directly
   if s then 
    possible_playstate=s
   end
   -- random playstate from ocular sensor
   if ((playstate==6) and (playstate_counter==0) and s5) then 
    possible_playstate=math.random(1,5) 
   end
   -- lock playstate now
   if (possible_playstate~=playstate) then
    playstate_wanted=possible_playstate
    if (io_debug==1) then
     message="Playstate "..playstate_wanted.." ..."
    end
    if (playstate_counter==0) then 
     playstate_counter=fade_scale*rate 
    end
    -- enable parallel port D1 (LED)
    interface.data=3
    -- reset timeout counter since there is activity
    timeout_counter=0
    rtimeout_counter=0
   else
    -- disable parallel port D1 (LED)
    interface.data=1 
    -- increase timeout counter
    if (playstate~=6) then
     timeout_counter=timeout_counter+1
     if (timeout_counter>=timeout) then
      timeout_counter=0
      -- go to attract loop state 6 = F
      playstate_wanted=6
      if (playstate_counter==0) then 
       playstate_counter=fade_scale*rate 
      end   
     end
    end
   end 
  else 
   -- disable parallel port D1 (LED)
   interface.data=1    
   -- increase timeout counter
   if (playstate~=6) then
    timeout_counter=timeout_counter+1
    if (timeout_counter>=timeout) then
     timeout_counter=0
     -- go to attract loop state 6 = F
     playstate_wanted=6
     if (playstate_counter==0) then 
      playstate_counter=fade_scale*rate 
     end   
    end
   end
   -- increase second timeout counter
   if (playstate==6) then
    rtimeout_counter=rtimeout_counter+1
    if (rtimeout_counter>rtimeout) then
     -- rewind all movies to first frame
     -- TODO
    end
   end 
  end
  last_seconds=seconds
 end
end

function Done()
 print "Shutdown Section"
end
